#include <stdlib.h>

#include <uemf.h>
#include <websda.h>
#include <wsIntrn.h>

#include <cyg/hal/iic.h>
#include "../dm.h"
#include "../ident.h"
#include "../state.h"

#include <cyg/hal/plf_stub.h>

/*  
 *  Stubs for missing functions when SocketServer is not included.  
 *  Should find a new way to do this.
 */
#ifdef NO_SOCKETSERVER
static int getTcpKeepaliveTimeout(void)
{
  return 0;
}

static int serverSaveConfig(void)
{
    return 0;
}
static void setTcpKeepaliveTimeout(int timeout)
{
    return;
}

void serverWipeConfig(void)
{
    return;
}

#endif

/* stringize a macro value -- trust me, it really has to be done this way */  
#define __String(a) #a
#define String(a) __String(a)  

#if !defined(__ECOS)
#define diag_printf printf
#endif

extern tServerState serverState[];
extern char versionString[];
extern int sprintf(char *buf, const char *fmt, ...);

extern int serverSaveConfig(void);
extern void cyg_select_abort(void);

typedef struct 
{
  int val;
  char *str;
}tEnumDef;

tEnumDef interfaceMap[] =
{
  {Mode232, "RS-232"},
  {Mode422, "RS-422"},
  {Mode485, "RS-485"},
  {-1,NULL}
};

tEnumDef parityMap[] =
{
  {ParityNone, "none"},
  {ParityEven, "even"},
  {ParityOdd,  "odd"},
  {-1,NULL}
};

tEnumDef flowMap[] =
{
  {FlowNone, "none"},
  {FlowRTSCTS, "RTS/CTS"},
  {FlowXonXoff, "Xon/Xoff"},
  {FlowHalfDuplex, "HalfDuplex"},
  {-1,NULL}
};

tEnumDef dtrMap[] =
{
  {DTRon, "on"},
  {DTRoff, "off"},
  {DTRconnected, "socketon"},
  {-1,NULL}
};

static int strToEnum(tEnumDef *p, char *s)
{
  while (p->str)
    {
      if (!strcmp(p->str,s))
        return p->val;
      ++p;
    }
  return -1;
}

static char *enumToStr(tEnumDef *p, int v)
{
  while (p->str)
    {
      if (v == p->val)
        return p->str;
      ++p;
    }
  return NULL;
}

static char digits[] = "0123456789ABCDEF";

char *utoa(unsigned i)
{
  static char buf[16];
  char *p = buf+(sizeof buf)-2;
  if (i==0)
    *p-- = '0';
  else
    while (i && p>=buf)
      {
        *p-- = digits[i%10];
        i /= 10;
      }
  return p+1;
}

char *itoa(int i)
{
  static char buf[16];
  char *p = buf+(sizeof buf)-2;
  int neg = i<0;
  if (neg)
    i = -i;
  if (i==0)
    *p-- = '0';
  else
    {
      while (i && p>=buf)
        {
          *p-- = digits[i%10];
          i /= 10;
        }
      if (neg)
        *p-- = '-';
    }
  return p+1;
}

static char *IPtoStr(unsigned n)
{
  static char buf[16];
  strcpy(buf,utoa(n>>24 & 0xff));
  strcat(buf,".");
  strcat(buf,utoa(n>>16 & 0xff));
  strcat(buf,".");
  strcat(buf,utoa(n>>8 & 0xff));
  strcat(buf,".");
  strcat(buf,utoa(n>>0 & 0xff));
  return buf;
}


unsigned long strToIP(char *s)
{
  unsigned long r=0;
  char *p = s;
  while (*p)
    {
      r <<= 8;
      r |= strtol(p,&p,10);
      if (*p)
        ++p;
    }
  return r;
}

/*----------------------------------------------------------------------
 * helper stuff
 */

static int validPort(int argc, char **argv)
{
  int p;
  
  if (argc != 1)
    return -1;

  p = atoi(argv[0]);
  
  if (p<0 || p>=ident.numPorts)
    return -1;
  else
    return p;
}

/*
 * eJscript functions
 */

extern unsigned long myIpAddr(void);
extern unsigned long myIpMask(void);
extern unsigned long myIpGate(void);

static char ZeroString[] = "0";
static char OneString[] = "1";

static int aspLocal_getModelIsAir(int eid, webs_t wp, int argc, char_t **argv)
{
  extern int ModelIsAir(void);
  ejSetResult(eid, utoa(ModelIsAir()));
  return 1;
}

static int aspLocal_getTcpKeepaliveTimeout(int eid, webs_t wp, int argc, char_t **argv)
{
  extern int getTcpKeepaliveTimeout(void);
  ejSetResult(eid, utoa(getTcpKeepaliveTimeout()));
  return 1;
}

static int aspLocal_getMyIdentIpAddr(int eid, webs_t wp, int argc, char_t **argv)
{
  ejSetResult(eid, IPtoStr(ident.ipAddr));
  return 1;
}

static int aspLocal_getMyIpAddr(int eid, webs_t wp, int argc, char_t **argv)
{
  ejSetResult(eid, IPtoStr(myIpAddr()));
  return 1;
}

static int aspLocal_getMyNetMask(int eid, webs_t wp, int argc, char_t **argv)
{
  ejSetResult(eid, IPtoStr(myIpMask()));
  return 1;
}

static int aspLocal_getMyGateway(int eid, webs_t wp, int argc, char_t **argv)
{
  ejSetResult(eid, IPtoStr(myIpGate()));
  return 1;
}

static int aspLocal_getCfgIpAddr(int eid, webs_t wp, int argc, char_t **argv)
{
  unsigned long addr,gate,mask;
  getIpInfoRt(&addr,&mask,&gate);
  ejSetResult(eid, IPtoStr(addr));
  return 1;
}

static int aspLocal_getCfgNetMask(int eid, webs_t wp, int argc, char_t **argv)
{
  unsigned long addr,gate,mask;
  getIpInfoRt(&addr,&mask,&gate);
  ejSetResult(eid, IPtoStr(mask));
  return 1;
}

static int aspLocal_getCfgGateway(int eid, webs_t wp, int argc, char_t **argv)
{
  unsigned long addr,gate,mask;
  getIpInfoRt(&addr,&mask,&gate);
  ejSetResult(eid, IPtoStr(gate));
  return 1;
}

static int aspLocal_modelNumber(int eid, webs_t wp, int argc, char_t **argv)
{
  ejSetResult(eid, utoa(ident.modelId));
  return 1;
}

static int aspLocal_versionString(int eid, webs_t wp, int argc, char_t **argv)
{
  ejSetResult(eid, versionString);
  return 1;
}

static int aspLocal_numPorts(int eid, webs_t wp, int argc, char_t **argv)
{
  ejSetResult(eid, utoa(ident.numPorts));
  return 1;
}

static int aspLocal_getPortEnable(int eid, webs_t wp, int argc, char_t **argv)
{
  int p = validPort(argc,argv);
  if (p<0)
    return 0;
  ejSetResult(eid, utoa(serverState[p].config.enabled));
  return 1;
}

static int aspLocal_getLocalPort(int eid, webs_t wp, int argc, char_t **argv)
{
  int p = validPort(argc,argv);
  if (p<0)
    return 0;
  ejSetResult(eid, utoa(serverState[p].locPort));
  return 1;
}

static int aspLocal_getRemotePort(int eid, webs_t wp, int argc, char_t **argv)
{
  int p = validPort(argc,argv);
  if (p<0)
    return 0;
  ejSetResult(eid, utoa(serverState[p].remPort));
  return 1;
}

static int aspLocal_getLocalIP(int eid, webs_t wp, int argc, char_t **argv)
{
  int p = validPort(argc,argv);
  if (p<0)
    return 0;
  ejSetResult(eid, IPtoStr(serverState[p].locIP));
  return 1;
}

static int aspLocal_getRemoteIP(int eid, webs_t wp, int argc, char_t **argv)
{
  int p = validPort(argc,argv);
  if (p<0)
    return 0;
  ejSetResult(eid, IPtoStr(serverState[p].remIP));
  return 1;
}

static int aspLocal_getTxCount(int eid, webs_t wp, int argc, char_t **argv)
{
  int p = validPort(argc,argv);
  if (p<0)
    return 0;
  ejSetResult(eid, utoa(serverState[p].txCount));
  return 1;
}

static int aspLocal_getRxCount(int eid, webs_t wp, int argc, char_t **argv)
{
  int p = validPort(argc,argv);
  if (p<0)
    return 0;
  ejSetResult(eid, utoa(serverState[p].rxCount));
  return 1;
}

static int aspLocal_getBaudRate(int eid, webs_t wp, int argc, char_t **argv)
{
  int p = validPort(argc,argv);
  if (p<0)
    return 0;
  ejSetResult(eid, utoa(serverState[p].config.baudRate));
  return 1;
}

static int aspLocal_getIcTimeout(int eid, webs_t wp, int argc, char_t **argv)
{
  int p = validPort(argc,argv);
  if (p<0)
    return 0;
  ejSetResult(eid, utoa(serverState[p].config.icTimeout));
  return 1;
}

static int aspLocal_getWordLength(int eid, webs_t wp, int argc, char_t **argv)
{
  int p = validPort(argc,argv);
  if (p<0)
    return 0;
  ejSetResult(eid, utoa(serverState[p].config.wordLength));
  return 1;
}

static int aspLocal_getStopBits(int eid, webs_t wp, int argc, char_t **argv)
{
  int p = validPort(argc,argv);
  if (p<0)
    return 0;
  ejSetResult(eid, utoa(serverState[p].config.stopBits));
  return 1;
}

static int aspLocal_getFlowControl(int eid, webs_t wp, int argc, char_t **argv)
{
  int p = validPort(argc,argv);
  if (p<0)
    return 0;
  ejSetResult(eid, enumToStr(flowMap, serverState[p].config.flowControl));
  return 1;
}

static int aspLocal_getInterfaceMode(int eid, webs_t wp, int argc, char_t **argv)
{
  int p = validPort(argc,argv);
  if (p<0)
    return 0;
  ejSetResult(eid, enumToStr(interfaceMap, serverState[p].config.interfaceMode));
  return 1;
}

static int aspLocal_getParity(int eid, webs_t wp, int argc, char_t **argv)
{
  int p = validPort(argc,argv);
  if (p<0)
    return 0;
  ejSetResult(eid, enumToStr(parityMap, serverState[p].config.parity));
  return 1;
}

static int aspLocal_getDtrControl(int eid, webs_t wp, int argc, char_t **argv)
{
  int p = validPort(argc,argv);
  if (p<0)
    return 0;
  ejSetResult(eid, enumToStr(dtrMap, serverState[p].config.dtrControl));
  return 1;
}

static int aspLocal_getListenEnable(int eid, webs_t wp, int argc, char_t **argv)
{
  int p = validPort(argc,argv);
  if (p<0)
    return 0;
  ejSetResult(eid, utoa(serverState[p].config.listenEnable));
  return 1;
}

static int aspLocal_getListenPort(int eid, webs_t wp, int argc, char_t **argv)
{
  int p = validPort(argc,argv);
  if (p<0)
    return 0;
  ejSetResult(eid, utoa(serverState[p].config.listenPort));
  return 1;
}

static int aspLocal_getTelnetEnable(int eid, webs_t wp, int argc, char_t **argv)
{
  int p = validPort(argc,argv);
  if (p<0)
    return 0;
  ejSetResult(eid, utoa(serverState[p].config.telnetEnable));
  return 1;
}

static int aspLocal_getConnectToCondData(int eid, webs_t wp, int argc, char_t **argv)
{
  int p = validPort(argc,argv);
  if (p<0)
    return 0;
  ejSetResult(eid, serverState[p].config.connectOnData ? OneString : ZeroString);
  return 1;
}

static int aspLocal_getConnectToCondAlways(int eid, webs_t wp, int argc, char_t **argv)
{
  int p = validPort(argc,argv);
  if (p<0)
    return 0;
  ejSetResult(eid, serverState[p].config.connectAlways ? OneString : ZeroString);
  return 1;
}

static int aspLocal_getConnectToCondDSR(int eid, webs_t wp, int argc, char_t **argv)
{
  int p = validPort(argc,argv);
  if (p<0)
    return 0;
  ejSetResult(eid, serverState[p].config.connectOnDSR ? OneString : ZeroString);
  return 1;
}

static int aspLocal_getConnectToCondCD(int eid, webs_t wp, int argc, char_t **argv)
{
  int p = validPort(argc,argv);
  if (p<0)
    return 0;
  ejSetResult(eid, serverState[p].config.connectOnCD ? OneString : ZeroString);
  return 1;
}

static int aspLocal_getConnectToCond(int eid, webs_t wp, int argc, char_t **argv)
{
  static char buf[32];
  static char *s;
  int p = validPort(argc,argv);
  if (p<0)
    return 0;

  s = buf;
  *s = '\0';

  if (serverState[p].config.connectAlways)
    strcat(buf,"always");
  else
    {
      if (serverState[p].config.connectOnData)
        strcat(buf," data ");
      if (serverState[p].config.connectOnCD)
        strcat(buf," CD ");
      if (serverState[p].config.connectOnDSR)
        strcat(buf," DSR ");
      
      if (buf[0] == '\0')
        strcat(buf,"never");
    }
  
  ejSetResult(eid, buf);
  return 1;
}

static int aspLocal_getConnectToIP(int eid, webs_t wp, int argc, char_t **argv)
{
  int p = validPort(argc,argv);

  if (p<0)
    return 0;
  ejSetResult(eid, IPtoStr(serverState[p].config.connectToIP));
  return 1;
}

static int aspLocal_getConnectToPort(int eid, webs_t wp, int argc, char_t **argv)
{
  int p = validPort(argc,argv);
  if (p<0)
    return 0;
  ejSetResult(eid, utoa(serverState[p].config.connectToPort));
  return 1;
}

static int aspLocal_getDisconnCond(int eid, webs_t wp, int argc, char_t **argv)
{
  static char buf[32];
  static char *s;
  int p = validPort(argc,argv);
  if (p<0)
    return 0;

  s = buf;
  *s = '\0';
  
  if (serverState[p].config.disconnOnIdle)
    strcat(buf," idle ");
  if (serverState[p].config.disconnOnNoCD)
    strcat(buf," noCD ");
  if (serverState[p].config.disconnOnNoDSR)
    strcat(buf," noDSR ");
  
  if (buf[0] == '\0')
    strcat(buf,"never");
  
  ejSetResult(eid, buf);
  return 1;
}

static int aspLocal_getDisconnCondNoDSR(int eid, webs_t wp, int argc, char_t **argv)
{
  int p = validPort(argc,argv);
  if (p<0)
    return 0;
  ejSetResult(eid, serverState[p].config.disconnOnNoDSR ? OneString : ZeroString);
  return 1;
}

static int aspLocal_getDisconnCondIdle(int eid, webs_t wp, int argc, char_t **argv)
{
  int p = validPort(argc,argv);
  if (p<0)
    return 0;
  ejSetResult(eid, serverState[p].config.disconnOnIdle ? OneString : ZeroString);
  return 1;
}

static int aspLocal_getDisconnCondNoCD(int eid, webs_t wp, int argc, char_t **argv)
{
  int p = validPort(argc,argv);
  if (p<0)
    return 0;
  ejSetResult(eid, serverState[p].config.disconnOnNoCD ? OneString : ZeroString);
  return 1;
}

static int aspLocal_getDisconnIdleTime(int eid, webs_t wp, int argc, char_t **argv)
{
  int p = validPort(argc,argv);
  if (p<0)
    return 0;
  ejSetResult(eid, utoa(serverState[p].config.idleTimeout));
  return 1;
}

//===============

/*
 * Form handlers
 */

static void setPortConfig(webs_t wp, char_t *path, char_t *query)
{
  unsigned long i;
  int n,p,start,end;
  char_t	*s;
  tServerState *state;
  
  if ((s=websGetVar(wp,"portNum", NULL)))
    p = atoi(s);
  else
    return;

  if (websGetVar(wp,"clonePort",NULL) || p < 0)
    {
      start = 0;
      end = DevMastMaxPorts;
    }
  else
    {
      start = p;
      end = p+1;
    }
  
  if (start<0 || end >DevMastMaxPorts)
    return;

  for (n=start; n<end; ++n)
    {
      state = serverState+n;
  
      cyg_mutex_lock(&state->serverMutex);
      state->config.enabled = !!websGetVar(wp,"portEnable",NULL);
      
      if ((s=websGetVar(wp,"baudRate", NULL)))
        state->config.baudRate = atoi(s);
      
      if ((s=websGetVar(wp,"interfaceMode", NULL)))
        state->config.interfaceMode = strToEnum(interfaceMap,s);
      
      if ((s=websGetVar(wp,"parity", NULL)))
        state->config.parity = strToEnum(parityMap,s);
      
      if ((s=websGetVar(wp,"icTimeout", NULL)))
        state->config.icTimeout = atoi(s);
      
      if ((s=websGetVar(wp,"flowControl", NULL)))
        state->config.flowControl = strToEnum(flowMap,s);
      
      if ((s=websGetVar(wp,"wordLength", NULL)))
        state->config.wordLength = i=atoi(s);
      
      if ((s=websGetVar(wp,"stopBits", NULL)))
        state->config.stopBits = i=atoi(s);
      
      if ((s=websGetVar(wp,"dtrControl", NULL)))
        state->config.dtrControl = strToEnum(dtrMap,s);
      
      state->config.listenEnable = !!websGetVar(wp,"listenEnable", NULL);

      state->config.telnetEnable = !!websGetVar(wp,"telnetEnable", NULL);
     
      if ((n==p) && (s=websGetVar(wp,"listenPort", NULL)))
        state->config.listenPort = atoi(s);
    
      if ((n==p) && (s=websGetVar(wp,"connectToIP", NULL)))
        state->config.connectToIP = strToIP(s);
      
      if ((n==p) && (s=websGetVar(wp,"connectToPort", NULL)))
        state->config.connectToPort = atoi(s);
          
      state->config.connectAlways = !!websGetVar(wp,"connectAlways", NULL);
      state->config.connectOnData = !!websGetVar(wp,"connectOnData", NULL);
      state->config.connectOnDSR  = !!websGetVar(wp,"connectOnDSR", NULL);
      state->config.connectOnCD   = !!websGetVar(wp,"connectOnCD", NULL);
      
      state->config.disconnOnIdle  = !!websGetVar(wp,"disconnOnIdle", NULL);
      state->config.disconnOnNoCD  = !!websGetVar(wp,"disconnOnNoCD", NULL);
      state->config.disconnOnNoDSR = !!websGetVar(wp,"disconnOnNoDSR", NULL);
        
      if ((s=websGetVar(wp,"idleTime", NULL)))
        state->config.idleTimeout = atoi(s);
      cyg_mutex_unlock(&state->serverMutex);
    }
  
  // wake up threads who need to know about the change
  cyg_select_abort();
  serverSaveConfig();
  websRedirect(wp,"/portSaved.asp");
}

int passwordOK(webs_t wp)
{
  char *requestPassword = websGetRequestPassword(wp);
  int flags = websGetRequestFlags(wp);

  if (!ident.password[0])
    return 1;
  
  if (requestPassword && !strcmp(requestPassword,ident.password))
    return 1;
  
  if (requestPassword && (flags & WEBS_AUTH_DIGEST))
    {
      char *digestCalc;
      wp->password = ident.password;
      a_assert(wp->digest);
      a_assert(wp->nonce);
      a_assert(wp->password);
      digestCalc = websCalcDigest(wp);
      a_assert(digestCalc);
      if (!strcmp(wp->digest, digestCalc))
        return 1;
    }
  websError(wp,401,"Password Required");
  return 0;
}

static void setNetConfig(webs_t wp, char_t *path, char_t *query)
{
  char_t  *s;
  extern void setTcpKeepaliveTimeout(int);
  unsigned ipAddr = 0;
  unsigned ipMask = 0;
  unsigned ipGate = 0;
 
  if (!passwordOK(wp))
    return;
  
  if (!(s=websGetVar(wp,"addrcfg", NULL)))
    return;

  if (!strcmp(s,"DHCP"))
    ipAddr = 0x00000000;
  else if (!strcmp(s,"Disable"))
    ipAddr = 0xffffffff;
  else if ((s=websGetVar(wp,"address",NULL)))
    ipAddr = strToIP(s);

  if ((s=websGetVar(wp,"gateway", NULL)))
    ipGate = strToIP(s);

  if ((s=websGetVar(wp,"netmask", NULL)))
    ipMask = strToIP(s);

  if ((s=websGetVar(wp,"timeout", NULL)))
    setTcpKeepaliveTimeout(atoi(s));

  putIpInfoRt(ipAddr,ipMask,ipGate);

  serverSaveConfig();
  websRedirect(wp,"/netSaved.asp");
}

static void resetUnit(webs_t wp, char_t *path, char_t *query)
{
  void serverWipeConfig(void);
  int doWipe;
  if (!passwordOK(wp))
    return;
  doWipe = !!websGetVar(wp,"factoryDefaults", NULL);
  websHeader(wp);
  websWrite(wp, "<head><META HTTP-EQUIV=\"REFRESH\" CONTENT=\"15;URL=/home.asp\"></head>\n");
  websWrite(wp, "<hr>\n");
  websWrite(wp, "<h1 align=center>DeviceMaster Resetting</h1>\n");
  if (doWipe)
    websWrite(wp,"<br><p>[restoring port configurations to factory defaults]</p>\n");
  websWrite(wp, "<p>Will redirect to <a href=\"/home.asp\">/home.asp</a> in 15 seconds.</p>\n");
  websWrite(wp, "<hr>\n");
  websFooter(wp);
  websDone(wp,200);
  if (doWipe)
    serverWipeConfig();
  cyg_thread_delay(50);
  HAL_STUB_PLATFORM_RESET();  
}


/*============================================================================
 * The only visible routine
 */

void initLocalSupport(void)
{
  websAspDefine(T("getModelIsAir"), aspLocal_getModelIsAir);

  websAspDefine(T("modelNumber"), aspLocal_modelNumber);
  websAspDefine(T("versionString"), aspLocal_versionString);
  
  websAspDefine(T("myIdentIpAddr"), aspLocal_getMyIdentIpAddr);
  
  websAspDefine(T("myIpAddr"), aspLocal_getMyIpAddr);
  websAspDefine(T("myNetMask"), aspLocal_getMyNetMask);
  websAspDefine(T("myGateway"), aspLocal_getMyGateway);

  websAspDefine(T("cfgIpAddr"), aspLocal_getCfgIpAddr);
  websAspDefine(T("cfgNetMask"), aspLocal_getCfgNetMask);
  websAspDefine(T("cfgGateway"), aspLocal_getCfgGateway);

  websAspDefine(T("myTcpTimeout"), aspLocal_getTcpKeepaliveTimeout);
  
  websAspDefine(T("numPorts"), aspLocal_numPorts);
  
  websAspDefine(T("getPortEnable"), aspLocal_getPortEnable);
  websAspDefine(T("getLocalIP"), aspLocal_getLocalIP);
  websAspDefine(T("getLocalPort"), aspLocal_getLocalPort);
  websAspDefine(T("getRemoteIP"), aspLocal_getRemoteIP);
  websAspDefine(T("getRemotePort"), aspLocal_getRemotePort);
  websAspDefine(T("getRxCount"), aspLocal_getRxCount);
  websAspDefine(T("getTxCount"), aspLocal_getTxCount);
  
  websAspDefine(T("getInterfaceMode"), aspLocal_getInterfaceMode);
  websAspDefine(T("getBaudRate"), aspLocal_getBaudRate);
  websAspDefine(T("getWordLength"), aspLocal_getWordLength);
  websAspDefine(T("getStopBits"), aspLocal_getStopBits);
  websAspDefine(T("getFlowControl"), aspLocal_getFlowControl);
  websAspDefine(T("getParity"), aspLocal_getParity);
  websAspDefine(T("getDtrControl"), aspLocal_getDtrControl);
  websAspDefine(T("getListenEnable"), aspLocal_getListenEnable);
  websAspDefine(T("getListenPort"), aspLocal_getListenPort);
  websAspDefine(T("getTelnetEnable"), aspLocal_getTelnetEnable);
  websAspDefine(T("getIcTimeout"), aspLocal_getIcTimeout);


  websAspDefine(T("getConnectToCond"), aspLocal_getConnectToCond);
  websAspDefine(T("getConnectToCondAlways"), aspLocal_getConnectToCondAlways);
  websAspDefine(T("getConnectToCondData"), aspLocal_getConnectToCondData);
  websAspDefine(T("getConnectToCondDSR"), aspLocal_getConnectToCondDSR);
  websAspDefine(T("getConnectToCondCD"), aspLocal_getConnectToCondCD);
  
  websAspDefine(T("getConnectToIP"), aspLocal_getConnectToIP);
  websAspDefine(T("getConnectToPort"), aspLocal_getConnectToPort);

  
  websAspDefine(T("getDisconnCond"), aspLocal_getDisconnCond);
  websAspDefine(T("getDisconnCondIdle"), aspLocal_getDisconnCondIdle);
  websAspDefine(T("getDisconnCondNoCD"), aspLocal_getDisconnCondNoCD);
  websAspDefine(T("getDisconnCondNoDSR"), aspLocal_getDisconnCondNoDSR);
  websAspDefine(T("getDisconnIdleTime"), aspLocal_getDisconnIdleTime);

  websFormDefine(T("setPortConfig"), setPortConfig);

  websFormDefine(T("setNetConfig"), setNetConfig);
  websFormDefine(T("resetUnit"), resetUnit);
}

char_t *basicGetHost(void)
{
  diag_printf("STUB: basicGetHost()\n");
  return "";
}
